@@ -19,6 +19,12 @@
     
 */
 
+/* WARNING: Don't even think about adding support
+   for ffmpeg's PCM codecs because they will crash.
+   
+*/
+
+#include "config.h"
 #include <ctype.h>
 #include <string.h>
 #include <quicktime/lqt.h>
@@ -27,6 +33,7 @@
 #include "ffmpeg.h"
 
 #define MAX_FOURCCS 30
+#define MAX_WAV_IDS 4
 
 int ffmpeg_num_audio_codecs = -1;
 int ffmpeg_num_video_codecs = -1;
@@ -42,52 +49,12 @@ int ffmpeg_num_video_codecs = -1;
           (char**)0\
 	}
 
-#define ENCODE_PARAM_VIDEO \
-	{\
-	  "bit_rate",\
-          "Bit rate (kbps)",\
-	  LQT_PARAMETER_INT,\
-          { 800 },\
-          0,\
-          0,\
-          (char**)0\
-	},\
-  	{\
-          "bit_rate_tolerance",\
-          "Bit rate tolernce",\
-          LQT_PARAMETER_INT,\
-          { 1024 * 8 },\
-          4,\
-          24000,\
-          (char**)0\
-	},\
-	{\
-          "flags_fix",\
-          "Fixed quality (VBR)",\
-          LQT_PARAMETER_INT,\
-          { 0 },\
-          0,\
-          1,\
-          (char**)0\
-        },\
-	{\
-          "quality",\
-          "VBR Quality",\
-          LQT_PARAMETER_INT,\
-          { 1 },\
-          1,\
-          31,\
-          (char**)0\
-	},\
-	{\
-          "flags_hq",\
-          "High quality (non realtime) mode",\
-          LQT_PARAMETER_INT,\
-          { 0 },\
-          0,\
-          1,\
-          (char**)0\
-        },\
+#define ENCODE_PARAM_VIDEO_GENERAL \
+        { \
+        "general", \
+        "General", \
+        LQT_PARAMETER_SECTION, \
+        }, \
         {\
           "flags_gray",\
           "Gray scale only mode",\
@@ -96,185 +63,216 @@ int ffmpeg_num_video_codecs = -1;
           0,\
           1,\
           (char**)0\
+        }, \
+	{ \
+	"strict_std_compliance", \
+	"Standard compilance", \
+	LQT_PARAMETER_INT, \
+	{ 0 }, \
+	0, \
+	2, \
+	(char**)0 \
+	}, \
+        {  \
+        "aspect_ratio_info", \
+        "Aspect Ratio", \
+	LQT_PARAMETER_STRINGLIST, \
+        {val_string:  "Square" }, \
+        0, \
+        0, \
+        (char*[]){ "Square", "4:3", "16:9", (char*)0 } \
         }
   
-#define ENCODE_PARAM_MOTION_EST \
+#define ENCODE_PARAM_VIDEO_BITRATE \
+        { \
+        "bitrate", \
+        "Bitrate", \
+        LQT_PARAMETER_SECTION, \
+        }, \
+        {\
+	  "bit_rate",\
+          "Bit rate (kbps)",\
+	  LQT_PARAMETER_INT,\
+          { 800 },\
+          0,\
+          0,\
+          (char**)0\
+	},\
+        { \
+          "bit_rate_tolerance",\
+          "Bitrate Tolerance (kbps)\n",\
+          LQT_PARAMETER_INT,\
+          { 4000 } \
+        }, \
+        { \
+          "rc_min_rate",\
+          "Minimum bitrate (kbps)\n",\
+          LQT_PARAMETER_INT,\
+          { 0 } \
+        }, \
+        { \
+          "rc_max_rate",\
+          "Maximum bitrate (kbps)\n",\
+          LQT_PARAMETER_INT,\
+          { 0 } \
+        }, \
+        { \
+        "qcompress", \
+        "Qscale change between easy and hard scenes", \
+        LQT_PARAMETER_INT, \
+        { 50 }, \
+        0, \
+        100, \
+        (char**)0 \
+        }, \
+        { \
+        "qblur", \
+        "Qscale smoothing over time", \
+        LQT_PARAMETER_INT, \
+        { 50 }, \
+        0, \
+        100, \
+        (char**)0 \
+        }
+  
+#define ENCODE_PARAM_VIDEO_VBR \
+        { \
+        "vbr", \
+        "VBR Options", \
+        LQT_PARAMETER_SECTION, \
+        }, \
+        {\
+          "qscale", \
+          "VBR quantizer scale (0 = CBR)", \
+          LQT_PARAMETER_INT, \
+          { 0 }, \
+          0, \
+          31 \
+        }, \
+        {\
+          "qmin", \
+          "min quantiser scale (VBR)", \
+          LQT_PARAMETER_INT, \
+          { 2 }, \
+          0, \
+          31 \
+        }, \
+        {\
+          "qmax", \
+          "max quantiser scale (VBR)", \
+          LQT_PARAMETER_INT, \
+          { 31 }, \
+          0, \
+          31 \
+        }, \
+        {\
+          "mb_qmin", \
+          "min macroblock quantiser scale (VBR)", \
+          LQT_PARAMETER_INT, \
+          { 2 }, \
+          0, \
+          31 \
+        }, \
+        {\
+          "mb_qmax", \
+          "max macroblock quantiser scale (VBR)", \
+          LQT_PARAMETER_INT, \
+          { 31 }, \
+          0, \
+          31 \
+        }, \
+        {\
+          "qdiff", \
+          "max difference between the quantiser scale (VBR)", \
+          LQT_PARAMETER_INT, \
+          { 3 }, \
+          0, \
+          31 \
+        } \
+
+#define ENCODE_PARAM_VIDEO_TEMPORAL \
+        { \
+        "temporal_compression", \
+        "Temporal Compression", \
+        LQT_PARAMETER_SECTION \
+        }, \
 	{\
-		"me_method",\
-		"Motion estimation method",\
-		LQT_PARAMETER_STRINGLIST,\
-		{val_string: "Zero"},\
-		0,\
-		100,\
-		((char *[]){"Zero", "Phods", "Log", "X1", "Epzs", "Full", (char *)0})\
+	"me_method",\
+	"Motion estimation method",\
+	LQT_PARAMETER_STRINGLIST,\
+	{val_string: "Zero"},\
+        0, \
+        0, \
+	((char *[]){"Zero", "Phods", "Log", "X1", "Epzs", "Full", (char *)0})\
 	},\
 	{\
-		"flags_4mv",\
-		"4MV mode",\
-		LQT_PARAMETER_INT,\
-		{ 0 },\
-		0,\
-		1,\
-		(char**)0\
-	}
-  
-#define ENCODE_PARAM_MPEGVIDEO \
-  { \
-    "gop_size", \
-    "GOP size", \
-    LQT_PARAMETER_INT, \
-    { 250 }, \
-    0, \
-    300, \
-    (char**)0 \
-  }, \
-  { \
-    "aspect_ratio_info",\
-    "Aspect ratio",\
-    LQT_PARAMETER_STRINGLIST,\
-    {val_string: "Square"},\
-    0,\
-    100,\
-    ((char *[]){"Square", "4:3 (625)", "4:3 (525)",\
-                "16:9 (625)", "16:9 (525)", (char *)0})\
-  },\
-  { \
-    "luma_elim_threshold",\
-    "Luma elimination threshold",\
-    LQT_PARAMETER_INT,\
-    { 0 },\
-    0,\
-    99,\
-    (char**)0\
-  },\
-  { \
-    "chroma_elim_threshold",\
-    "Chroma elimination threshold",\
-    LQT_PARAMETER_INT,\
-    { 0 },\
-    0,\
-    99,\
-    (char**)0 \
-  }, \
-  { \
-    "flags_part", \
-    "Data partitioning mode", \
-    LQT_PARAMETER_INT, \
-    { 0 }, \
-    0, \
-    1, \
-    (char**)0 \
-    }, \
-    { \
-    "flags_pass", \
-    "Pass (0 = single pass)", \
-    LQT_PARAMETER_INT, \
-    { 0 }, \
-    0, \
-    2, \
-    (char**)0 \
-  }, \
-  { \
-    "max_b_frames", \
-    "Maximum B frames", \
-    LQT_PARAMETER_INT, \
-    { 0 }, \
-    0, \
-    FF_MAX_B_FRAMES, \
-    (char**)0 \
-  }, \
-  { \
-    "b_frame_strategy", \
-    "B Frame strategy", \
-    LQT_PARAMETER_INT, \
-    { 0 }, \
-    0, \
-    1, \
-    (char**)0 \
-  }, \
-  { \
-    "b_quant_factor", \
-    "Scale factor btw IPS and B frames", \
-    LQT_PARAMETER_INT, \
-    { 2 }, \
-    0, \
-    31, \
-    (char**)0 \
-  }, \
-  { \
-    "b_quant_offset", \
-    "Scale offset btw IPS and B frames", \
-    LQT_PARAMETER_INT, \
-    { 0 }, \
-    0, \
-    31, \
-    (char**)0 \
-  }, \
-  { \
-    "qcompress", \
-    "Qscale change between easy and hard scenes", \
-    LQT_PARAMETER_INT, \
-    { 50 }, \
-    0, \
-    100, \
-    (char**)0 \
-  }, \
-  { \
-    "qblur", \
-    "Qscale smoothing over time", \
-    LQT_PARAMETER_INT, \
-    { 50 }, \
-    0, \
-    100, \
-    (char**)0 \
-  }, \
-  { \
-    "qmin", \
-    "Minimum qscale", \
-    LQT_PARAMETER_INT, \
-    { 3 }, \
-    1, \
-    31, \
-    (char**)0 \
-  }, \
-  { \
-    "qmax", \
-    "Maximum qscale", \
-    LQT_PARAMETER_INT, \
-    { 15 }, \
-    1, \
-    31, \
-    (char**)0 \
-  }, \
-  { \
-    "max_qdiff", \
-    "Maximum qscale difference", \
-    LQT_PARAMETER_INT, \
-    { 3 }, \
-    1, \
-    31, \
-    (char**)0 \
-  }, \
-  { \
-    "rc_strategy", \
-    "RC strategy", \
-    LQT_PARAMETER_INT, \
-    { 2 }, \
-    0, \
-    2, \
-    (char**)0 \
-  }, \
-  { \
-    "rtp_payload_size", \
-    "Packet size (0 = variable)", \
-    LQT_PARAMETER_INT, \
-    { 0 }, \
-    0, \
-    1000000, \
-    (char**)0 \
-  }
+	"mb_decision",\
+	"MB decision mode",\
+	LQT_PARAMETER_STRINGLIST,\
+	{val_string: "Simple"},\
+        0, \
+        0, \
+	((char *[]){"Simple", "Fewest bits", "Rate distoration", (char *)0})\
+	},\
+        { \
+        "gop_size", \
+        "GOP size (0 = intra only)", \
+        LQT_PARAMETER_INT, \
+        { val_int: 250 }, \
+        0, \
+        300, \
+        (char**)0 \
+        } \
+ 
+#define ENCODE_PARAM_VIDEO_MPEG4 \
+        { \
+        "mpeg4", \
+        "MPEG-4 Options", \
+        LQT_PARAMETER_SECTION \
+        }, \
+	{\
+	"flags_4mv",\
+	"Use four motion vector by macroblock",\
+	LQT_PARAMETER_INT,\
+	{ 0 },\
+	0,\
+	1,\
+	(char**)0\
+	}, \
+        { \
+        "flags_part", \
+        "Data partitioning mode", \
+        LQT_PARAMETER_INT, \
+        { 0 }, \
+        0, \
+        1, \
+        (char**)0 \
+        }
 
 
+#define ENCODE_PARAM_VIDEO_H263P \
+        { \
+        "h263p", \
+        "H263+ Options", \
+        LQT_PARAMETER_SECTION \
+        }, \
+        { \
+        "flags_h263p_aic", \
+        "Advanced intra coding", \
+        LQT_PARAMETER_INT, \
+        { 0 }, \
+        0, \
+        1 \
+        }, \
+        { \
+        "flags_h263p_umv", \
+        "Unlimited Motion Vector", \
+        LQT_PARAMETER_INT, \
+        { 0 }, \
+        0, \
+        1 \
+        }
+
 #define DECODE_PARAM_AUDIO
 
 #define DECODE_PARAM_VIDEO \
@@ -295,34 +293,47 @@ int ffmpeg_num_video_codecs = -1;
           0,\
           1,\
           (char**)0\
+        }, \
+        { \
+          "error_resilience", \
+          "Error resilience", \
+          LQT_PARAMETER_STRINGLIST, \
+          { val_string: "Careful" }, \
+          0, \
+          0, \
+          (char *[]){"None", "Careful", "Compilant", "Agressive", "Very Agressive", (char *)0 } \
         }
-  
+
 static lqt_parameter_info_static_t encode_parameters_video[] = {
-  ENCODE_PARAM_VIDEO,
+  ENCODE_PARAM_VIDEO_GENERAL,
+  ENCODE_PARAM_VIDEO_BITRATE,
+  ENCODE_PARAM_VIDEO_VBR,
   { /* End of parameters */ }
 };
 
 static lqt_parameter_info_static_t encode_parameters_mpegvideo[] = {
-  ENCODE_PARAM_VIDEO,
-  ENCODE_PARAM_MOTION_EST,
-  ENCODE_PARAM_MPEGVIDEO,
+  ENCODE_PARAM_VIDEO_GENERAL,
+  ENCODE_PARAM_VIDEO_BITRATE,
+  ENCODE_PARAM_VIDEO_VBR,
+  ENCODE_PARAM_VIDEO_TEMPORAL,
   { /* End of parameters */ }
 };
 
 static lqt_parameter_info_static_t encode_parameters_mpeg4[] = {
-  ENCODE_PARAM_VIDEO,
-  ENCODE_PARAM_MOTION_EST,
-  ENCODE_PARAM_MPEGVIDEO,
-  // Encoding mpeg4
-	{
-		"strict_std_compliance",
-		"Comply strictly with standards",
-		LQT_PARAMETER_INT,
-		{ 0 },
-		0,
-		1,
-		(char**)0
-	},
+  ENCODE_PARAM_VIDEO_GENERAL,
+  ENCODE_PARAM_VIDEO_BITRATE,
+  ENCODE_PARAM_VIDEO_VBR,
+  ENCODE_PARAM_VIDEO_TEMPORAL,
+  ENCODE_PARAM_VIDEO_MPEG4,
+  { /* End of parameters */ }
+};
+
+static lqt_parameter_info_static_t encode_parameters_h263p[] = {
+  ENCODE_PARAM_VIDEO_GENERAL,
+  ENCODE_PARAM_VIDEO_BITRATE,
+  ENCODE_PARAM_VIDEO_VBR,
+  ENCODE_PARAM_VIDEO_TEMPORAL,
+  ENCODE_PARAM_VIDEO_H263P,
   { /* End of parameters */ }
 };
 
@@ -339,15 +350,11 @@ static lqt_parameter_info_static_t decode_parameters_video[] = {
 
 static lqt_parameter_info_static_t decode_parameters_mpeg4[] = {
   DECODE_PARAM_VIDEO,
-	{
-		"error_resilience",
-		"Error resilience",
-		LQT_PARAMETER_INT,
-		{ 1 },
-		0,
-		1,
-		(char**)0
-	},
+  { /* End of parameters */ }
+};
+
+static lqt_parameter_info_static_t decode_parameters_rle[] = {
+  DECODE_PARAM_VIDEO,
   { /* End of parameters */ }
 };
 
@@ -367,6 +374,7 @@ struct CODECIDMAP {
 	char *short_name;
 	char *name;
 	char *fourccs[MAX_FOURCCS];
+        int   wav_ids[MAX_WAV_IDS];
 };
 
 struct CODECIDMAP codecidmap_v[] = {
@@ -423,8 +431,8 @@ struct CODECIDMAP codecidmap_v[] = {
           decode_parameters: decode_parameters_mpeg4,
 	  short_name: "msmpeg4v3",
 	  name: "MSMpeg 4v3",
-	  fourccs: {"MPG3", "mpg3", "MP43", "mp43", "DIV5", "div5", "DIV6",
-                    "div6", "DIV3", "div3", "DIV4", "div4", "AP41", "ap41",
+	  fourccs: {"DIV3", "mpg3", "MP43", "mp43", "DIV5", "div5", "DIV6",
+                    "MPG3", "div6", "div3", "DIV4", "div4", "AP41", "ap41",
                     (char *)0}
         },
 #if 0
@@ -454,7 +462,7 @@ struct CODECIDMAP codecidmap_v[] = {
 	  index: -1,
           encoder: NULL,
           decoder: NULL,
-          encode_parameters: encode_parameters_mpegvideo,
+          encode_parameters: encode_parameters_h263p,
           decode_parameters: decode_parameters_video,
 	  short_name: "h263p",
 	  name: "H263+",
@@ -481,6 +489,26 @@ struct CODECIDMAP codecidmap_v[] = {
 	  name: "Real Video 10",
 	  fourccs: {"RV10", "rv10", "RV13", "rv13", (char *)0} },
 #endif
+	{
+          id: CODEC_ID_SVQ1,
+	  index: -1,
+          encoder: NULL,
+          decoder: NULL,
+          encode_parameters: encode_parameters_mpegvideo,
+          decode_parameters: decode_parameters_video,
+	  short_name: "svq1",
+	  name: "Sorenson Video 1",
+	  fourccs: {"SVQ1", (char *)0} },
+	{
+          id: CODEC_ID_SVQ3,
+	  index: -1,
+          encoder: NULL,
+          decoder: NULL,
+          encode_parameters: encode_parameters_mpegvideo,
+          decode_parameters: decode_parameters_video,
+	  short_name: "svq3",
+	  name: "Sorenson Video 3",
+	  fourccs: {"SVQ3", (char *)0} },
 	{
           id: CODEC_ID_MJPEG,
 	  index: -1,
@@ -491,6 +519,76 @@ struct CODECIDMAP codecidmap_v[] = {
 	  short_name: "mjpg",
 	  name: "MJPEG",
 	  fourccs: {"MJPG", "mjpg", "JPEG", "jpeg", "dmb1", (char *)0} },
+	{
+          id: CODEC_ID_8BPS,
+	  index: -1,
+          encoder: NULL,
+          decoder: NULL,
+          encode_parameters: encode_parameters_video,
+          decode_parameters: decode_parameters_video,
+	  short_name: "8BPS",
+	  name: "Quicktime Planar RGB (8BPS)",
+	  fourccs: {"8BPS", (char *)0} },
+	{
+          id: CODEC_ID_INDEO3,
+	  index: -1,
+          encoder: NULL,
+          decoder: NULL,
+          encode_parameters: encode_parameters_video,
+          decode_parameters: decode_parameters_video,
+	  short_name: "indeo",
+	  name: "Intel Indeo 3",
+	  fourccs: {"IV31", "IV32", (char *)0} },
+	{
+          id: CODEC_ID_RPZA,
+	  index: -1,
+          encoder: NULL,
+          decoder: NULL,
+          encode_parameters: encode_parameters_video,
+          decode_parameters: decode_parameters_video,
+	  short_name: "rpza",
+	  name: "Apple Video",
+	  fourccs: {"rpza", (char *)0} },
+	{
+          id: CODEC_ID_SMC,
+	  index: -1,
+          encoder: NULL,
+          decoder: NULL,
+          encode_parameters: encode_parameters_video,
+          decode_parameters: decode_parameters_video,
+	  short_name: "smc",
+	  name: "Apple Graphics",
+	  fourccs: {"smc ", (char *)0} },
+	{
+          id: CODEC_ID_CINEPAK,
+	  index: -1,
+          encoder: NULL,
+          decoder: NULL,
+          encode_parameters: encode_parameters_video,
+          decode_parameters: decode_parameters_video,
+	  short_name: "cinepak",
+	  name: "Cinepak",
+	  fourccs: {"cvid", (char *)0} },
+	{
+          id: CODEC_ID_CYUV,
+	  index: -1,
+          encoder: NULL,
+          decoder: NULL,
+          encode_parameters: encode_parameters_video,
+          decode_parameters: decode_parameters_video,
+	  short_name: "cyuv",
+	  name: "Creative YUV",
+	  fourccs: {"CYUV", (char *)0} },
+	{
+          id: CODEC_ID_QTRLE,
+	  index: -1,
+          encoder: NULL,
+          decoder: NULL,
+          encode_parameters: encode_parameters_video,
+          decode_parameters: decode_parameters_video,
+	  short_name: "rle",
+	  name: "RLE",
+	  fourccs: {"rle ", (char *)0} },
 };
 
 struct CODECIDMAP codecidmap_a[] = {
@@ -504,7 +602,9 @@ struct CODECIDMAP codecidmap_a[] = {
           decode_parameters: decode_parameters_audio,
 	  short_name: "mp2",
 	  name: "Mpeg Layer 2 Audio",
-	  {".mp2", ".MP2", "ms\0\x50", "MS\0\x50", (char *)0} },
+	  fourccs: {".mp2", ".MP2", "ms\0\x50", "MS\0\x50", (char *)0},
+          wav_ids: { LQT_WAV_ID_NONE },
+        },
 	{
           id: CODEC_ID_MP3LAME,
 	  index: -1,
@@ -514,7 +614,9 @@ struct CODECIDMAP codecidmap_a[] = {
           decode_parameters: decode_parameters_audio,
 	  short_name: "mp3",
 	  name: "Mpeg Layer 3 Audio",
-	  {".mp3", ".MP3", "ms\0\x55", "MS\0\x55"} },
+	  fourccs: {".mp3", ".MP3", "ms\0\x55", "MS\0\x55", (char *)0},
+          wav_ids: { 0x55, LQT_WAV_ID_NONE },
+        },
 	{
           id: CODEC_ID_AC3,
 	  index: -1,
@@ -524,7 +626,31 @@ struct CODECIDMAP codecidmap_a[] = {
           decode_parameters: decode_parameters_audio,
 	  short_name: "ac3",
 	  name: "AC3 Audio",
-	  {".ac3", ".AC3", (char *)0} },
+	  fourccs: {".ac3", ".AC3", (char *)0},
+          wav_ids: { 0x2000, LQT_WAV_ID_NONE },
+        },
+#if 0 /* Crashes */
+	{
+          id: CODEC_ID_ADPCM_MS,
+	  index: -1,
+          encoder: NULL,
+          decoder: NULL,
+          short_name: "adpcm (ms)",
+	  name: "McRowsoft ADPCM",
+	  fourccs: {"ms\0\x02", "MS\0\x02", (char*)0},
+          wav_ids: { 0x02, LQT_WAV_ID_NONE },
+        },
+#endif
+	{
+          id: CODEC_ID_PCM_ALAW,
+	  index: -1,
+          encoder: NULL,
+          decoder: NULL,
+          short_name: "alaw",
+	  name: "Alaw",
+	  fourccs: { "alaw", (char*)0},
+          wav_ids: { 0x06, LQT_WAV_ID_NONE },
+        },
 };
 
 
@@ -532,50 +658,38 @@ struct CODECIDMAP codecidmap_a[] = {
 #define NUMMAPS_V ((int)(sizeof(codecidmap_v)/sizeof(struct CODECIDMAP)))
 
 void ffmpeg_map_init(void)
-{
-	AVCodec *codec;
-	int i, found;
-	if(ffmpeg_num_video_codecs >= 0)
-		return;
-	avcodec_register_all();
-	avcodec_init();
-	ffmpeg_num_video_codecs = 0;
-	ffmpeg_num_audio_codecs = 0;
-	for(codec = first_avcodec; codec; codec = codec->next) {
-                found = 0;
-		for(i = 0; i < NUMMAPS_V; i++) {
-			if(codec->id == codecidmap_v[i].id) {
-				if(codecidmap_v[i].index < 0)
-					codecidmap_v[i].index =
-                                          ffmpeg_num_audio_codecs +
-                                          ffmpeg_num_video_codecs++;
-				if(codec->encode)
-					codecidmap_v[i].encoder = codec;
-				if(codec->decode)
-					codecidmap_v[i].decoder = codec;
-                                found = 1;
-				break;
-			}
-		}
-                if(!found) {
-         	  for(i = 0; i < NUMMAPS_A; i++) {
-			if(codec->id == codecidmap_a[i].id) {
-				if(codecidmap_a[i].index < 0)
-                                        codecidmap_a[i].index =
-                                          ffmpeg_num_video_codecs +
-                                          ffmpeg_num_audio_codecs++;
-				if(codec->encode)
-					codecidmap_a[i].encoder = codec;
-				if(codec->decode)
-					codecidmap_a[i].decoder = codec;
-                                found = 1;
-				break;
-			}
-                  }
-                }
-                
-	}
-}
+  {
+  AVCodec *codec;
+  int i, found;
+  if(ffmpeg_num_video_codecs >= 0)
+    return;
+  avcodec_register_all();
+  avcodec_init();
+  ffmpeg_num_video_codecs = 0;
+  ffmpeg_num_audio_codecs = 0;
+  
+  for(i = 0; i < NUMMAPS_V; i++)
+    {
+    codecidmap_v[i].encoder = avcodec_find_encoder(codecidmap_v[i].id);
+    codecidmap_v[i].decoder = avcodec_find_decoder(codecidmap_v[i].id);
+
+    if(codecidmap_v[i].encoder || codecidmap_v[i].decoder)
+      codecidmap_v[i].index = ffmpeg_num_audio_codecs + ffmpeg_num_video_codecs++;
+    }
+  for(i = 0; i < NUMMAPS_A; i++)
+    {
+    /* UGLY HACK: Disable alaw encoding since it screws up libquicktime */
+    if(codecidmap_a[i].id != CODEC_ID_PCM_ALAW)
+      codecidmap_a[i].encoder = avcodec_find_encoder(codecidmap_a[i].id);
+    codecidmap_a[i].decoder = avcodec_find_decoder(codecidmap_a[i].id);
+
+    if(codecidmap_a[i].encoder || codecidmap_a[i].decoder)
+      {
+      codecidmap_a[i].index = ffmpeg_num_audio_codecs++ + ffmpeg_num_video_codecs;
+      // fprintf(stderr, "Found codec %s %p %p\n", codecidmap_a[i].name, codecidmap_a[i].encoder, codecidmap_a[i].decoder);
+      }
+    }
+  }
 
 /* Common to all */
 static int encoding_colormodels_ffmpeg[] = {
@@ -594,12 +708,13 @@ static lqt_codec_info_static_t codec_info_ffmpeg = {
 	long_name:   ffmpeg_long_name,
 	description: ffmpeg_description,
 	fourccs:     NULL,
+        wav_ids:     NULL,
 	type:        0,
 	direction:   0,
 	encoding_parameters: NULL,
 	decoding_parameters: NULL,
 	encoding_colormodels: encoding_colormodels_ffmpeg,
-	decoding_colormodel:  BC_YUV420P
+	decoding_colormodel:  LQT_COLORMODEL_NONE,
 };
 
 /* These are called from the plugin loader */
@@ -612,36 +727,41 @@ extern int get_num_codecs()
 
 static void set_codec_info(struct CODECIDMAP * map)
   {
-  char * capabilities = (char*)0;
+  // char * capabilities = (char*)0;
 
   codec_info_ffmpeg.fourccs = map->fourccs;
+  codec_info_ffmpeg.wav_ids = map->wav_ids;
 
   if(map->encoder && map->decoder)
     {
     codec_info_ffmpeg.direction = LQT_DIRECTION_BOTH;
     codec_info_ffmpeg.encoding_parameters = map->encode_parameters;
     codec_info_ffmpeg.decoding_parameters = map->decode_parameters;
-    capabilities = "Codec";
+    //    capabilities = "Codec";
     }
   else if(map->encoder)
     {
     codec_info_ffmpeg.direction = LQT_DIRECTION_ENCODE;
     codec_info_ffmpeg.encoding_parameters = map->encode_parameters;
     codec_info_ffmpeg.decoding_parameters = NULL;
-    capabilities = "Encoder";
+    //    capabilities = "Encoder";
     }
   else if(map->decoder)
     {
     codec_info_ffmpeg.direction = LQT_DIRECTION_DECODE;
     codec_info_ffmpeg.encoding_parameters = NULL;
     codec_info_ffmpeg.decoding_parameters = map->decode_parameters;
-    capabilities = "Decoder";
+    //    capabilities = "Decoder";
     }
 
   snprintf(ffmpeg_name, 50, "ffmpeg_%s", map->short_name);
-  snprintf(ffmpeg_long_name, 50, "FFMPEG %s %s", map->name, capabilities);
-  snprintf(ffmpeg_description, 100, "FFMPEG %s %s", map->name, capabilities);
+  //  snprintf(ffmpeg_long_name, 50, "FFMPEG %s %s", map->name, capabilities);
+  //  snprintf(ffmpeg_description, 100, "FFMPEG %s %s", map->name, capabilities);
 
+  snprintf(ffmpeg_long_name, 50, "FFMPEG %s", map->name);
+  snprintf(ffmpeg_description, 100, "FFMPEG %s", map->name);
+
+  
   if((map->encoder && (map->encoder->type == CODEC_TYPE_VIDEO)) ||
      (map->decoder && (map->decoder->type == CODEC_TYPE_VIDEO))){
        codec_info_ffmpeg.type = LQT_CODEC_VIDEO;
@@ -650,26 +770,81 @@ static void set_codec_info(struct CODECIDMAP * map)
   }
   }
 
+static struct CODECIDMAP * find_codec(int index)
+  {
+  int i;
+  for(i = 0; i < NUMMAPS_V; i++)
+    {
+    if(codecidmap_v[i].index == index)
+      return &codecidmap_v[i];
+    }
+  for(i = 0; i < NUMMAPS_A; i++)
+    {
+    if(codecidmap_a[i].index == index)
+      return &codecidmap_a[i];
+    }
+  return (struct CODECIDMAP *)0;
+  }
+
 extern lqt_codec_info_static_t * get_codec_info(int index)
 {
-	int i;
-	
-	ffmpeg_map_init();
-	for(i = 0; i < NUMMAPS_V; i++) {
-		if(codecidmap_v[i].index == index) {
-                set_codec_info(&codecidmap_v[i]);
-                return &codec_info_ffmpeg;
-		}
-	}
-	for(i = 0; i < NUMMAPS_A; i++) {
-                if(codecidmap_a[i].index == index) {
-                set_codec_info(&codecidmap_a[i]);
-                return &codec_info_ffmpeg;
-		}
-	}
+	struct CODECIDMAP * map;
+        ffmpeg_map_init();
+	map = find_codec(index);
+
+        if(map)
+          {
+          set_codec_info(map);
+          return &codec_info_ffmpeg;
+          }
 	return NULL;
 }
 
+/* We obtain the stream colormodel during runtime */
+
+extern int get_stream_colormodel(quicktime_t * file, int track, int codec_index,
+                                 int * exact)
+  {
+  int width, height, depth, ret;
+  quicktime_video_map_t *vtrack;
+  quicktime_trak_t *trak;
+  AVCodecContext *avctx;
+  int dummy;
+  struct CODECIDMAP * map;
+    
+  vtrack = &(file->vtracks[track]);
+  trak = vtrack->track;
+  
+  height = trak->tkhd.track_height;
+  width = trak->tkhd.track_width;
+  depth = quicktime_video_depth(file, track);
+
+  avctx = avcodec_alloc_context();
+
+  avctx->width = width;
+  avctx->height = height;
+  avctx->bits_per_sample = depth;
+  avctx->extradata      = trak->mdia.minf.stbl.stsd.table[0].extradata;
+  avctx->extradata_size = trak->mdia.minf.stbl.stsd.table[0].extradata_size;
+
+  map = find_codec(codec_index);
+  if(!map)
+    {
+    fprintf(stderr, "Found no codec for %d\n", codec_index);
+    return LQT_COLORMODEL_NONE;
+    }
+  if(avcodec_open(avctx, map->decoder) != 0)
+    {
+    fprintf(stderr, "Couldn't open codec No. %d\n", codec_index);
+    return LQT_COLORMODEL_NONE;
+    }
+  ret = lqt_ffmpeg_get_lqt_colormodel(avctx->pix_fmt, &dummy);
+  
+  avcodec_close(avctx);
+  return ret;
+  }
+
+
 /*
  *   Return the actual codec constructor
  */
@@ -721,8 +896,8 @@ IFUNC(26)
 IFUNC(27)
 IFUNC(28)
 IFUNC(29)
+#undef IFUNC
 #define MAX_VIDEO_FUNC 29
-
 #define IFUNC(x) \
 void quicktime_init_audio_codec_ffmpeg ## x(quicktime_audio_map_t *atrack) \
 { \
@@ -857,11 +1032,3 @@ extern lqt_init_audio_codec_func_t get_audio_codec(int index)
 	}
 	return (lqt_init_audio_codec_func_t)0;
 }
-
-
-
-int get_stream_colormodel(quicktime_t * file, int track, int codec_index,
-                          int * exact)
-{
-	return BC_YUV420P;
-}
