@@ -1,5 +1,6 @@
 // Divx for encore50
 
+#include "config.h"
 #include <quicktime/colormodels.h>
 #include <funcprotos.h>
 #include <quicktime/quicktime.h>
@@ -191,7 +192,7 @@ static void putbits(unsigned char **data,
 	int count, 
 	uint64_t value)
 {
-	value &= 0xffffffffffffffff >> (64 - count);
+	value &= 0xffffffffffffffffLL >> (64 - count);
 
 	while(64 - *bit_pos < count)
 	{
@@ -233,7 +234,7 @@ int quicktime_divx_write_vol(unsigned char *data_start,
 	int vol_width, 
 	int vol_height, 
 	int time_increment_resolution, 
-	float frame_rate)
+	double frame_rate)
 {
 	int written = 0;
 	int bits, fixed_vop_time_increment;
@@ -582,7 +583,7 @@ static int decode(quicktime_t *file, unsigned char **row_pointers, int track)
 
 
 	input_cmodel = BC_YUV420P;
-	if(file->color_model == input_cmodel &&
+	if(file->vtracks[track].color_model == input_cmodel &&
 		file->out_w == width_i &&
 		file->out_h == height_i &&
 		file->in_x == 0 &&
@@ -731,7 +732,7 @@ static int decode(quicktime_t *file, unsigned char **row_pointers, int track)
 			file->out_w, 
 			file->out_h,
 			input_cmodel, 
-			file->color_model,
+			file->vtracks[track].color_model,
 			0,         /* When transfering BC_RGBA8888 to non-alpha this is the background color in 0xRRGGBB hex */
 			width_i,       /* For planar use the luma rowspan */
 			width);
@@ -762,6 +763,7 @@ static int encode(quicktime_t *file, unsigned char **row_pointers, int track)
 	int height_i = (int)((float)height / 16 + 0.5) * 16;
 	int result = 0;
 	int i;
+        quicktime_atom_t chunk_atom;
 	ENC_FRAME encore_input;
 	ENC_RESULT encore_result;
 //printf("encode 1 %d %d\n", width_i, height_i);
@@ -793,7 +795,7 @@ static int encode(quicktime_t *file, unsigned char **row_pointers, int track)
 //printf("encode 1\n");
 
 // Assume planes are contiguous
-	if(file->color_model == BC_YUV420P &&
+	if(file->vtracks[track].color_model == BC_YUV420P &&
 		width == width_i &&
 		height == height_i)
 	{
@@ -826,7 +828,7 @@ static int encode(quicktime_t *file, unsigned char **row_pointers, int track)
 			0, 
 			width, 
 			height,
-			file->color_model, 
+			file->vtracks[track].color_model, 
 			BC_YUV420P,
 			0,         /* When transfering BC_RGBA8888 to non-alpha this is the background color in 0xRRGGBB hex */
 			width,       /* For planar use the luma rowspan */
@@ -870,16 +872,9 @@ static int encode(quicktime_t *file, unsigned char **row_pointers, int track)
 		&encore_result);
 	pthread_mutex_unlock(&encode_mutex);
 //printf("encode 6\n");
-
+        quicktime_write_chunk_header(file, trak, &chunk_atom);
 	result = !quicktime_write_data(file, codec->work_buffer, encore_input.length);
-	quicktime_update_tables(file,
-						file->vtracks[track].track,
-						offset,
-						file->vtracks[track].current_chunk,
-						file->vtracks[track].current_position,
-						1,
-						encore_input.length);
-
+        quicktime_write_chunk_footer(file, trak, vtrack->current_chunk, &chunk_atom, 1);
 	file->vtracks[track].current_chunk++;
 
 	if(encore_result.isKeyFrame)
