@@ -284,9 +284,9 @@ int quicktime_init_acodec(quicktime_audio_map_t *atrack, int encode,
   lqt_init_audio_codec_func_t (*get_codec)(int);
     
   void * module;
-  
-  char *compressor = atrack->track->mdia.minf.stbl.stsd.table[0].format;
 
+  char *compressor = atrack->track->mdia.minf.stbl.stsd.table[0].format;
+  int wav_id       = atrack->track->mdia.minf.stbl.stsd.table[0].compression_id;
   atrack->codec = calloc(1, sizeof(quicktime_codec_t));
   quicktime_codec_defaults((quicktime_codec_t*)atrack->codec);
 
@@ -297,13 +297,23 @@ int quicktime_init_acodec(quicktime_audio_map_t *atrack, int encode,
   if(!codec_info)
     {
     
+
+    if(compressor && (*compressor != '\0'))
+      {
 #ifndef NDEBUG
     fprintf(stderr, "Trying to find Audio %s for fourcc \"%s\"...",
             (encode ? "Encoder" : "Decoder"), compressor);
 #endif
-    
-    codec_array = lqt_find_audio_codec(compressor, encode);
-    
+      codec_array = lqt_find_audio_codec(compressor, encode);
+      }
+    else if(wav_id)
+      {
+#ifndef NDEBUG
+      fprintf(stderr, "Trying to find Audio %s for wav_id 0x%02x...",
+              (encode ? "Encoder" : "Decoder"), wav_id);
+#endif
+      codec_array = lqt_find_audio_codec_by_wav_id(wav_id, encode);
+      }
     if(!codec_array)
       {
 #ifndef NDEBUG
@@ -371,6 +381,12 @@ int quicktime_init_acodec(quicktime_audio_map_t *atrack, int encode,
   
   init_codec(atrack);
 
+  /* We set the wav ids from our info structure, so we don't have to do this
+     in the plugin sources */
+
+  if(codec_info->num_wav_ids)
+    ((quicktime_codec_t*)((quicktime_codec_t*)atrack->codec))->wav_id = codec_info->wav_ids[0];
+  
   if(codec_array)
     lqt_destroy_codec_info(codec_array);
   return 0;
@@ -430,17 +446,39 @@ int quicktime_supported_video(quicktime_t *file, int track)
 
 int quicktime_supported_audio(quicktime_t *file, int track)
 {
-	char *compressor = quicktime_audio_compressor(file, track);
-        lqt_codec_info_t ** test_codec =
-          lqt_find_audio_codec(compressor, file->wr);
-        
-        if(!test_codec)
-          return 0;
+         lqt_codec_info_t ** test_codec;
+         char *compressor = quicktime_audio_compressor(file, track);
+
+         test_codec = (lqt_codec_info_t**)0;
+         
+         if(compressor && (*compressor != '\0'))
+           test_codec = lqt_find_audio_codec(compressor, file->wr);
+         else if(lqt_is_avi(file))
+           test_codec = lqt_find_audio_codec_by_wav_id(lqt_get_wav_id(file, track), file->wr);
+         
+         if(!test_codec)
+           return 0;
         
         lqt_destroy_codec_info(test_codec);
         return 1;
 }
 
+static void update_frame_position(quicktime_video_map_t * track)
+  {
+  track->timestamp +=
+    track->track->mdia.minf.stbl.stts.table[track->stts_index].sample_duration;
+
+  track->stts_count++;
+
+  if(track->stts_count >=
+     track->track->mdia.minf.stbl.stts.table[track->stts_index].sample_count)
+    {
+    track->stts_index++;
+    track->stts_count = 0;
+    }
+  track->current_position++;
+  }
+
 int quicktime_decode_video(quicktime_t *file,
                            unsigned char **row_pointers, int track)
 {
@@ -452,7 +490,8 @@ int quicktime_decode_video(quicktime_t *file,
 //printf("quicktime_decode_video 1\n");
 // Fake scaling parameters
 	file->do_scaling = 0;
-	file->color_model = BC_RGB888;
+                
+        file->vtracks[track].color_model = BC_RGB888;
 	file->in_x = 0;
 	file->in_y = 0;
 	file->in_w = track_width;
@@ -460,9 +499,9 @@ int quicktime_decode_video(quicktime_t *file,
 	file->out_w = track_width;
 	file->out_h = track_height;
 
-//printf("quicktime_decode_video 1\n");
+        //printf("quicktime_decode_video 1\n");
 	result = ((quicktime_codec_t*)file->vtracks[track].codec)->decode_video(file, row_pointers, track);
-	file->vtracks[track].current_position++;
+        update_frame_position(&(file->vtracks[track]));
 //printf("quicktime_decode_video 2\n");
 	return result;
 }
@@ -492,7 +531,7 @@ int lqt_decode_video(quicktime_t *file,
 //printf("quicktime_decode_video 1\n");
 	result =
           ((quicktime_codec_t*)file->vtracks[track].codec)->decode_video(file, row_pointers, track);
-	file->vtracks[track].current_position++;
+        update_frame_position(&(file->vtracks[track]));
 //printf("quicktime_decode_video 2\n");
 	return result;
 }
@@ -511,7 +550,7 @@ long quicktime_decode_scaled(quicktime_t *file,
 	int result;
 
 	file->do_scaling = 1;
-	file->color_model = color_model;
+	file->vtracks[track].color_model = color_model;
 	file->in_x = in_x;
 	file->in_y = in_y;
 	file->in_w = in_w;
@@ -520,7 +559,7 @@ long quicktime_decode_scaled(quicktime_t *file,
 	file->out_h = out_h;
 
 	result = ((quicktime_codec_t*)file->vtracks[track].codec)->decode_video(file, row_pointers, track);
-	file->vtracks[track].current_position++;
+        update_frame_position(&(file->vtracks[track]));
 	return result;
 }
 
@@ -625,6 +664,54 @@ int lqt_decode_audio(quicktime_t *file,
 	return result;
 }
 
+int lqt_decode_audio_track(quicktime_t *file, 
+                           int16_t **poutput_i, 
+                           float **poutput_f, 
+                           long samples,
+                           int track)
+  {
+  int result = 1;
+  int16_t *output_i;
+  float *output_f;
+  int i;
+  int total_channels = quicktime_track_channels(file, track);
+  
+  for( i=0; i < total_channels; i++ )
+    {
+    
+    if( poutput_i != NULL )
+      {
+      output_i = poutput_i[i];
+      }
+    else
+      output_i = NULL;
+    
+    if( poutput_f != NULL )
+      {
+      output_f = poutput_f[i];
+      }
+    else
+      output_f = NULL;
+    
+    if( output_i != NULL || output_f != NULL )
+      {	
+      result = ((quicktime_codec_t*)file->atracks[track].codec)->decode_audio(
+                                                                              file, 
+                                                                              output_i, 
+                                                                              output_f, 
+                                                                              samples, 
+                                                                              track, 
+                                                                              i );
+      }
+    }
+  
+  file->atracks[track].current_position += samples;
+  
+  return result;
+  }
+
+
+
 /* Since all channels are written at the same time: */
 /* Encode using the compressor for the first audio track. */
 /* Which means all the audio channels must be on the same track. */
